package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.DefaultExportDTO;
import com.diagbot.dto.GatherDTO;
import com.diagbot.dto.IndicationNewExportDTO;
import com.diagbot.dto.KlRuleByIdDTO;
import com.diagbot.dto.KlRuleByIdParDTO;
import com.diagbot.dto.KlRuleByIdSubDTO;
import com.diagbot.dto.KlRuleInfoDTO;
import com.diagbot.dto.OtherExportDTO;
import com.diagbot.dto.RuleDTO;
import com.diagbot.dto.RuleQueryDTO;
import com.diagbot.entity.KlRule;
import com.diagbot.entity.KlRuleBase;
import com.diagbot.entity.KlRuleCondition;
import com.diagbot.enums.CommonEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.RedisEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.KlRuleServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ExtUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RedisUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.ExportDafaultVO;
import com.diagbot.vo.GatherVO;
import com.diagbot.vo.KlRuleByIdVO;
import com.diagbot.vo.KlRuleInfoClearVO;
import com.diagbot.vo.KlRuleInfoSaveSubVO;
import com.diagbot.vo.KlRuleInfoSaveVO;
import com.diagbot.vo.KlRuleInfoVO;
import com.diagbot.vo.KlRuleSatartOrdisaVO;
import com.diagbot.vo.RuleQueryKeyVO;
import com.diagbot.vo.RuleQueryVO;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @author: gaodm
 * @time: 2021/2/3 16:44
 */
@Component
public class KlRuleFacade extends KlRuleServiceImpl {
    @Autowired
    KlRuleConditionFacade klRuleConditionFacade;
    @Autowired
    KlRuleBaseFacade klRuleBaseFacade;
    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    KlConceptFacade klConceptFacade;

    /**
     * @param klRuleInfoVO
     * @return
     */
    public IPage<KlRuleInfoDTO> getKlRuleInfoPage(KlRuleInfoVO klRuleInfoVO) {
        return this.getKlRuleInfoPages(klRuleInfoVO);
    }

    public KlRuleByIdParDTO getByIdRuleInfos(KlRuleByIdVO klRuleByIdVO) {
        KlRuleByIdParDTO klRuleByIdPar = new KlRuleByIdParDTO();
        List<KlRuleByIdSubDTO> klRuleByIdSubLsit = new ArrayList<>();
        List<KlRuleByIdDTO> byIdRuleInfo = this.getByIdRuleInfo(klRuleByIdVO);
        if (ListUtil.isNotEmpty(byIdRuleInfo)) {
            klRuleByIdPar.setParId(byIdRuleInfo.get(0).getParId());
            klRuleByIdPar.setParDescription(byIdRuleInfo.get(0).getParDescription());
            klRuleByIdPar.setParRuleType(byIdRuleInfo.get(0).getParRuleType());
            klRuleByIdPar.setParConceptId(byIdRuleInfo.get(0).getParConceptId());
            klRuleByIdPar.setParlibName(byIdRuleInfo.get(0).getParlibName());
            klRuleByIdPar.setParHasSub(byIdRuleInfo.get(0).getParHasSub());//是否有子条件（0：无，1：有）
            klRuleByIdPar.setParMsg(byIdRuleInfo.get(0).getParMsg());//附加信息
            klRuleByIdPar.setParStatus(byIdRuleInfo.get(0).getParStatus());//启用状态（0：禁用，1：启用）
            klRuleByIdPar.setParLenName(byIdRuleInfo.get(0).getParLenName());
            klRuleByIdPar.setParLenCode(byIdRuleInfo.get(0).getParLenCode());
            for (KlRuleByIdDTO data : byIdRuleInfo) {
                if (null != data.getSubDescription()) {
                    KlRuleByIdSubDTO klRuleByIdSub = new KlRuleByIdSubDTO();
                    BeanUtil.copyProperties(data, klRuleByIdSub);
                    klRuleByIdSubLsit.add(klRuleByIdSub);
                }
            }
        }
        klRuleByIdPar.setKlRuleByIdSub(klRuleByIdSubLsit);
        return klRuleByIdPar;
    }

    public Boolean saveRuleInfos(KlRuleInfoSaveVO klRuleInfoSaveVO) {
        Date now = DateUtil.now();
        boolean res = false;
/*        // 校验名称是否相同
        int count = this.count(new QueryWrapper<KlRule>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("description", klRuleInfoSaveVO.getParDescription())
                .ne("id", klRuleInfoSaveVO.getParId() == null ? -1 : klRuleInfoSaveVO.getParId()));
        if (count > 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该规则名称已存在");
        }*/
        //校验数据是否还在
        if (null != klRuleInfoSaveVO.getParId()) {
            int sum = this.count(new QueryWrapper<KlRule>().eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("id", klRuleInfoSaveVO.getParId()));
            if (sum == 0) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该数据已不存在！");
            }
            UpdateWrapper<KlRule> klRuleUpdate = new UpdateWrapper<>();
            klRuleUpdate.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("id", klRuleInfoSaveVO.getParId())
                    .set(StringUtil.isNotBlank(klRuleInfoSaveVO.getParDescription()), "description", klRuleInfoSaveVO.getParDescription())
                    .set(StringUtil.isNotBlank(klRuleInfoSaveVO.getParConceptId().toString()), "concept_id", klRuleInfoSaveVO.getParConceptId())
                    .set(StringUtil.isNotBlank(klRuleInfoSaveVO.getParRuleType().toString()), "rule_type", klRuleInfoSaveVO.getParRuleType())
                    .set(StringUtil.isNotBlank(klRuleInfoSaveVO.getParHasSub().toString()), "has_sub_cond", klRuleInfoSaveVO.getParHasSub())
                    .set(StringUtil.isNotBlank(klRuleInfoSaveVO.getParStatus().toString()), "status", klRuleInfoSaveVO.getParStatus())
                    .set(StringUtil.isNotBlank(klRuleInfoSaveVO.getParMsg()), "msg", klRuleInfoSaveVO.getParMsg())
                    .set("modifier", klRuleInfoSaveVO.getModifier())
                    .set("gmt_modified", now);
            res = this.update(klRuleUpdate);

            //先删除原有的详情;
            res = clearSub(klRuleInfoSaveVO.getParId());
            if (ListUtil.isNotEmpty(klRuleInfoSaveVO.getKlRuleInfoSaveSub())) {
                //再重新传入的详情保存
                res = saveCommon(klRuleInfoSaveVO);
            }
        } else {
            //先保存主表rule
            KlRule klRule = new KlRule();
            klRule.setConceptId(klRuleInfoSaveVO.getParConceptId());
            klRule.setCreator(klRuleInfoSaveVO.getModifier());
            klRule.setModifier(klRuleInfoSaveVO.getModifier());
            klRule.setDescription(klRuleInfoSaveVO.getParDescription());
            klRule.setHasSubCond(klRuleInfoSaveVO.getParHasSub());
            klRule.setRuleType(klRuleInfoSaveVO.getParRuleType());
            klRule.setMsg(klRuleInfoSaveVO.getParMsg());
            klRule.setStatus(klRuleInfoSaveVO.getParStatus());
            klRule.setGmtCreate(now);
            klRule.setGmtModified(now);
            res = this.save(klRule);
            if (res) {
                //再保存附表
                klRuleInfoSaveVO.setParId(klRule.getId());
                res = saveCommon(klRuleInfoSaveVO);
            }
        }

        return res;
    }


    public Boolean clearRuleInfos(KlRuleInfoClearVO klRuleInfoClearVO) {
        boolean res = false;
        res = clearSub(klRuleInfoClearVO.getId());
        //最后删除主表rule
        res = this.removeById(klRuleInfoClearVO.getId());
        return res;
    }

    /**
     * 删除明细表
     */
    public boolean clearSub(Long id) {
        boolean res = false;
        QueryWrapper<KlRuleCondition> klRuleConditionQuery = new QueryWrapper<>();
        klRuleConditionQuery.eq("is_deleted", IsDeleteEnum.N.getKey()).eq("rule_id", id);
        List<KlRuleCondition> ruleConditionList = klRuleConditionFacade.list(klRuleConditionQuery);
        List<Long> ruleBaseId = new ArrayList<>();
        if (ListUtil.isNotEmpty(ruleConditionList)) {
            for (KlRuleCondition data : ruleConditionList) {
                ruleBaseId.add(data.getRuleBaseId());
            }
        }
        if (ListUtil.isNotEmpty(ruleBaseId)) {
            QueryWrapper<KlRuleBase> klRuleBaseQuery = new QueryWrapper<>();
            klRuleBaseQuery.eq("is_deleted", IsDeleteEnum.N.getKey()).in("id", ruleBaseId);
            //删除kl_rule_base表数据
            res = klRuleBaseFacade.remove(klRuleBaseQuery);

        }
        //删除kl_rule_condition表的数据
        res = klRuleConditionFacade.remove(klRuleConditionQuery);
        return res;
    }

    /**
     * 保存数据
     */
    public boolean saveCommon(KlRuleInfoSaveVO klRuleInfoSaveVO) {
        Date now = DateUtil.now();
        boolean res = false;
        List<KlRuleInfoSaveSubVO> klRuleInfoSaveSub = klRuleInfoSaveVO.getKlRuleInfoSaveSub();
        if (ListUtil.isNotEmpty(klRuleInfoSaveSub)) {
            //先保存数据到kl_rule_base表里
            List<KlRuleBase> klRuleBaseList = new ArrayList<>();
            for (KlRuleInfoSaveSubVO list : klRuleInfoSaveSub) {
                KlRuleCondition klRuleCondition = new KlRuleCondition();
                KlRuleBase klRuleBase = new KlRuleBase();
                klRuleBase.setGmtModified(now);
                klRuleBase.setModifier(klRuleInfoSaveVO.getModifier());
                klRuleBase.setConceptId(list.getSubConceptId());
                klRuleBase.setType(list.getSubType());
                klRuleBase.setDescription(list.getSubDescription());
                klRuleBase.setMaxOperator(list.getSubMaxOperator());
                klRuleBase.setMaxValue(list.getSubMaxValue());
                klRuleBase.setMaxUnit(list.getSubMaxUnit());
                klRuleBase.setMinOperator(list.getSubMinOperator());
                klRuleBase.setMinValue(list.getSubMinValue());
                klRuleBase.setMinUnit(list.getSubMinUnit());
                klRuleBase.setStatus(1);
                klRuleBase.setEqValue(list.getSubEqValue());
                klRuleBase.setEqOperator(list.getSubEqOperator());
                klRuleBase.setEqUnit(list.getSubEqUnit());
                res = klRuleBaseFacade.save(klRuleBase);
                //再把数据保存在kl_rule_condition
                if (res) {
                    klRuleCondition.setGmtCreate(now);
                    klRuleCondition.setModifier(klRuleInfoSaveVO.getModifier());
                    klRuleCondition.setGroupType(list.getGroupType());
                    klRuleCondition.setRuleBaseId(klRuleBase.getId());
                    klRuleCondition.setRuleId(klRuleInfoSaveVO.getParId());
                    klRuleCondition.setMsg(list.getSubMsg());
                    res = klRuleConditionFacade.save(klRuleCondition);
                }

            }
        }

        return res;
    }

    public Boolean startOrDisableRule(KlRuleSatartOrdisaVO klRuleSatartOrdisaVO, int start) {
        boolean res = false;
        Date now = DateUtil.now();
        int sum = this.count(new QueryWrapper<KlRule>().eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", klRuleSatartOrdisaVO.getId()));
        if (sum == 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该数据已不存在！");
        }
        UpdateWrapper<KlRule> klRuleUpdate = new UpdateWrapper<>();
        klRuleUpdate.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", klRuleSatartOrdisaVO.getId())
                .set("status", start)
                .set("modifier", klRuleSatartOrdisaVO.getModifier())
                .set("gmt_modified", now);
        return this.update(klRuleUpdate);
    }

    /**
     * 开单合理性导出
     *
     * @return
     */
    public void exportIndicationFac(HttpServletResponse response) {
        ExportDafaultVO exportDafaultVO = new ExportDafaultVO();
        exportDafaultVO.setRuleType(1);
        List<IndicationNewExportDTO> res = Lists.newArrayList();
        List<DefaultExportDTO> defaultExportDTOS = exportOther(exportDafaultVO);
        GatherVO gatherVO = new GatherVO();
        List<String> gatherNameList = Lists.newArrayList();
        for (DefaultExportDTO defaultExportDTO : defaultExportDTOS) {
            // 开单名称集合
            if (isGatherType(defaultExportDTO.getLibType())) {
                gatherNameList.add(defaultExportDTO.getLibName() + CommonEnum.splitSymbol.getName() + defaultExportDTO.getLibType());
            }
            // 规则集合
            if (isGatherType(defaultExportDTO.getBaseType())) {
                gatherNameList.add(defaultExportDTO.getBaseName() + CommonEnum.splitSymbol.getName() + defaultExportDTO.getBaseType());
            }
        }
        gatherVO.setGatherNameList(gatherNameList);
        Map<String, List<GatherDTO>> gatherMap = klConceptFacade.getGatherFac(gatherVO);
        Map<String, List<DefaultExportDTO>> multiKeyList = ExtUtil.getMultiKeyList(defaultExportDTOS, CommonEnum.splitSymbol.getName(), "ruleId", "groupType");
        for (String key : multiKeyList.keySet()) {
            List<DefaultExportDTO> defaultList = multiKeyList.get(key);
            for (DefaultExportDTO defaultExportDTO : defaultList) {
                // 集合类型对集合元素赋值
                if (isGatherType(defaultExportDTO.getBaseType())) {
                    List<GatherDTO> gatherDTOS = gatherMap.get(defaultExportDTO.getBaseName() + CommonEnum.splitSymbol.getName() + defaultExportDTO.getBaseType());
                    List<String> sonNameList = gatherDTOS.stream().map(r -> r.getSonName()).collect(Collectors.toList());
                    // 子元素赋值
                    defaultExportDTO.setSonName(StringUtils.join(sonNameList, CommonEnum.splitSymbol.getName()));
                }
            }
            DefaultExportDTO defaultExportDTO = defaultList.get(0);
            IndicationNewExportDTO indicationNewExportDTO = new IndicationNewExportDTO();
            indicationNewExportDTO.setRuleGroup(defaultExportDTO.getRuleGroup());
            indicationNewExportDTO.setLibName(defaultExportDTO.getLibName());
            indicationNewExportDTO.setLibTypeCn(defaultExportDTO.getLibTypeCn());
            indicationNewExportDTO.setMsg(defaultExportDTO.getMsg());
            indicationNewExportDTO.setDefaultExportDTOList(defaultList);
            // 集合类型对集合元素赋值
            if (isGatherType(defaultExportDTO.getLibType())) {
                List<GatherDTO> gatherDTOS = gatherMap.get(defaultExportDTO.getLibName() + CommonEnum.splitSymbol.getName() + defaultExportDTO.getLibType());
                List<String> sonNameList = gatherDTOS.stream().map(r -> r.getSonName()).collect(Collectors.toList());
                // 子元素赋值
                indicationNewExportDTO.setSonLibName(StringUtils.join(sonNameList, CommonEnum.splitSymbol.getName()));
            }
            res.add(indicationNewExportDTO);
        }
        ExcelUtils.exportExcel(res, null, "开单合理性", IndicationNewExportDTO.class, "indication.xlsx",
                response, 12.8f);
    }

    /**
     * 是否集合类型
     *
     * @param type
     * @return
     */
    public Boolean isGatherType(Integer type) {
        if (type == null) {
            return false;
        }
        return (type >= 308 && type <= 329) || (type >= 301 && type <= 304);
    }

    /**
     * 其他值提醒化验导出
     *
     * @return
     */
    public void exportOtherLis(HttpServletResponse response) {
        ExportDafaultVO exportDafaultVO = new ExportDafaultVO();
        exportDafaultVO.setRuleType(4);
        exportDafaultVO.setLibTypeList(Lists.newArrayList(108L));
        List<OtherExportDTO> otherExportDTOList = generateExport(exportDafaultVO);
        ExcelUtils.exportExcel(otherExportDTOList, null, "其他值提醒化验", OtherExportDTO.class, "otherLis.xlsx",
                response, 12.8F);
    }

    /**
     * 其他值提醒辅检导出
     *
     * @return
     */
    public void exportOtherPacs(HttpServletResponse response) {
        ExportDafaultVO exportDafaultVO = new ExportDafaultVO();
        exportDafaultVO.setRuleType(4);
        exportDafaultVO.setLibTypeList(Lists.newArrayList(112L));
        List<OtherExportDTO> otherExportDTOList = generateExport(exportDafaultVO);
        ExcelUtils.exportExcel(otherExportDTOList, null, "其他值提醒辅检", OtherExportDTO.class, "otherPacs.xlsx",
                response, 12.8F);
    }

    /**
     * 其他值提醒输血导出
     *
     * @return
     */
    public void exportOtherTransfusion(HttpServletResponse response) {
        ExportDafaultVO exportDafaultVO = new ExportDafaultVO();
        exportDafaultVO.setRuleType(5);
        exportDafaultVO.setLibTypeList(Lists.newArrayList(108L));
        List<OtherExportDTO> otherExportDTOList = generateExport(exportDafaultVO);
        ExcelUtils.exportExcel(otherExportDTOList, null, "其他值提醒输血", OtherExportDTO.class, "otherTransfusion.xlsx",
                response, 12.8F);
    }

    /**
     * 危急值化验导出
     *
     * @return
     */
    public void exportCriticalLis(HttpServletResponse response) {
        ExportDafaultVO exportDafaultVO = new ExportDafaultVO();
        exportDafaultVO.setRuleType(3);
        exportDafaultVO.setLibTypeList(Lists.newArrayList(108L));
        List<OtherExportDTO> otherExportDTOList = generateExport(exportDafaultVO);
        ExcelUtils.exportExcel(otherExportDTOList, null, "危急值化验", OtherExportDTO.class, "criticalLis.xlsx",
                response, 12.8F);
    }

    /**
     * 危急值辅检导出
     *
     * @return
     */
    public void exportCriticalPacs(HttpServletResponse response) {
        ExportDafaultVO exportDafaultVO = new ExportDafaultVO();
        exportDafaultVO.setRuleType(3);
        exportDafaultVO.setLibTypeList(Lists.newArrayList(112L));
        List<OtherExportDTO> otherExportDTOList = generateExport(exportDafaultVO);
        ExcelUtils.exportExcel(otherExportDTOList, null, "危急值辅检", OtherExportDTO.class, "criticalPacs.xlsx",
                response, 12.8F);
    }

    /**
     * 生成导出类
     *
     * @param exportDafaultVO
     * @return
     */
    public List<OtherExportDTO> generateExport(ExportDafaultVO exportDafaultVO) {
        // 导出类
        List<OtherExportDTO> otherExportDTOList = new ArrayList<>();
        List<DefaultExportDTO> defaultExportDTOS = exportOther(exportDafaultVO);
        Map<String, List<DefaultExportDTO>> multiKeyList = ExtUtil.getMultiKeyList(defaultExportDTOS, CommonEnum.splitSymbol.getName(), "ruleId", "groupType");
        for (String key : multiKeyList.keySet()) {
            List<DefaultExportDTO> defaultList = multiKeyList.get(key);
            DefaultExportDTO defaultExportDTO = defaultList.get(0);
            // rule中conceptId也是规则匹配的一部分，手动添加，判断依据是conceptId和baseConcept如果不一样的，就添加一条
            if (!defaultExportDTO.getConceptId().equals(defaultExportDTO.getBaseConcept())) {
                DefaultExportDTO addDefault = new DefaultExportDTO();
                BeanUtil.copyProperties(defaultExportDTO, addDefault);
                addDefault.setBaseConcept(defaultExportDTO.getConceptId());
                addDefault.setBaseName(defaultExportDTO.getLibName());
                addDefault.setBaseTypeCn(defaultExportDTO.getLibTypeCn());
                addDefault.setMinValue("");
                addDefault.setMinOperator("");
                addDefault.setMaxValue("");
                addDefault.setMaxOperator("");
                addDefault.setEqValue("");
                List<DefaultExportDTO> newDefaultList = Lists.newArrayList();
                newDefaultList.add(addDefault);
                newDefaultList.addAll(defaultList);
                multiKeyList.put(key, newDefaultList);
                defaultList = newDefaultList;
            }
            OtherExportDTO otherExportDTO = new OtherExportDTO();
            otherExportDTO.setRuleGroup(defaultExportDTO.getRuleGroup());
            otherExportDTO.setMsg(defaultExportDTO.getMsg());
            otherExportDTO.setDefaultExportDTOList(defaultList);
            otherExportDTOList.add(otherExportDTO);
        }
        return otherExportDTOList;
    }

    /**
     * 查询所有有效的规则
     *
     * @param ruleQueryVO
     * @return
     */
    public IPage<RuleQueryDTO> getRulePageFac(RuleQueryVO ruleQueryVO) {
        return this.getRulePage(ruleQueryVO);
    }

    /**
     * 根据录入内容查询有效规则的明细
     *
     * @param ruleQueryKeyVO
     * @return
     */
    public RuleDTO getRuleDetail(RuleQueryKeyVO ruleQueryKeyVO) {
        String key = RedisEnum.allRule.getName() + ruleQueryKeyVO.getLibType() + "_" + ruleQueryKeyVO.getLibName() + "_" + ruleQueryKeyVO.getRuleType();
        return redisUtil.get(key);
    }
}
