package com.diagbot.process;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.biz.push.entity.Scale;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleConditionDTO;
import com.diagbot.dto.RuleExtDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.RedisEnum;
import com.diagbot.enums.RuleTypeEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.facade.CommonFacade;
import com.diagbot.model.entity.PacsNum;
import com.diagbot.model.label.PacsLabel;
import com.diagbot.rule.AgeRule;
import com.diagbot.rule.CommonRule;
import com.diagbot.rule.GroupRule;
import com.diagbot.rule.LisRule;
import com.diagbot.rule.PacsRule;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.MsgUtil;
import com.diagbot.util.RedisUtil;
import com.diagbot.util.ReflectUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.ItemExt;
import com.diagbot.vo.RuleVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * @Description: 其他提示总入口
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class OtherTipProcess {

    @Autowired
    GroupRule groupRule;
    @Autowired
    CommonRule commonRule;
    @Autowired
    CommonFacade commonFacade;
    @Autowired
    LisRule lisRule;
    @Autowired
    PacsRule pacsRule;
    @Autowired
    RedisUtil redisUtil;
    @Autowired
    AgeRule ageRule;

    /**
     * 处理业务——化验
     *
     * @param wordCrfDTO
     * @param res
     */
    public void processLis(WordCrfDTO wordCrfDTO, IndicationDTO res) {
        List<Lis> lisList = wordCrfDTO.getLis();
        List<BillMsg> otherList = res.getOtherList();
        for (Lis lis : lisList) {
            RuleVO ruleVO = getRuleVO(lis, RuleTypeEnum.other.getKey());
            // 1、获取相关数据
            if (ruleVO != null) {
                RuleExtDTO ruleExtDTO = commonFacade.getRuleData(ruleVO);
                if (ruleExtDTO == null) {
                    continue;
                }
                List<RuleConditionDTO> ruleConditionDTOList = ruleExtDTO.getRuleConditionDTOList();
                for (RuleConditionDTO ruleConditionDTO : ruleConditionDTOList) {
                    List<RuleBaseDTO> ruleBaseDTOList = ruleConditionDTO.getRuleBaseDTOList();
                    // 2、规则匹配
                    lisRule.otherLis(lis, ruleBaseDTOList, otherList, wordCrfDTO, ruleConditionDTO.getMsg());
                }
            }
        }
    }

    /**
     * 处理业务——辅检
     *
     * @param wordCrfDTO
     * @param res
     */
    public void processPacs(WordCrfDTO wordCrfDTO, IndicationDTO res) {
        PacsLabel pacsLabel = wordCrfDTO.getPacsLabel();
        List<RuleVO> ruleList = getRuleVO(pacsLabel);
        List<BillMsg> otherList = res.getOtherList();
        for (RuleVO ruleVO : ruleList) {
            // 1、获取相关数据
            if (ruleVO != null) {
                RuleExtDTO ruleExtDTO = commonFacade.getRuleData(ruleVO);
                if (ruleExtDTO == null) {
                    continue;
                }
                List<RuleConditionDTO> ruleConditionDTOList = ruleExtDTO.getRuleConditionDTOList();
                for (RuleConditionDTO ruleConditionDTO : ruleConditionDTOList) {
                    List<RuleBaseDTO> ruleBaseDTOList = ruleConditionDTO.getRuleBaseDTOList();
                    // 2、规则匹配
                    pacsRule.otherPacs(ruleBaseDTOList, ruleConditionDTO.getMsg(), otherList, wordCrfDTO, ruleExtDTO);
                }
            }
        }

        // // TODO 后期删除 辅检数值型匹配
        // List<PacsNum> pacsNumList = pacsLabel.getPacsNumList();
        // for (PacsNum pacsNum : pacsNumList) {
        //     if ("心率".equals(pacsNum.getStandName())) {
        //         PD pd = pacsNum.getPd();
        //         if (pd != null && StringUtil.isNotBlank(pd.getValue())) {
        //             double v = Double.parseDouble(pd.getValue());
        //             String content = pacsNum.getStandName() + pd.getValue() + pd.getUnit();
        //             Double ageNum = wordCrfDTO.getAgeNum();
        //             Double min = null, max = null;
        //             String suggest = "";
        //             if (ageNum < 0.019) {
        //                 min = 110D;
        //                 max = 170D;
        //                 suggest = "年龄0-6天心率正常值为110-170bpm";
        //             } else if (ageNum < 0.164) {
        //                 min = 90D;
        //                 max = 160D;
        //                 suggest = "年龄7-59天心率正常值为90-160bpm";
        //             } else if (ageNum < 2) {
        //                 min = 90D;
        //                 max = 150D;
        //                 suggest = "年龄2-23月心率正常值为90-150bpm";
        //             } else if (ageNum < 3) {
        //                 min = 80D;
        //                 max = 140D;
        //                 suggest = "年龄2-3岁心率正常值为80-140bpm";
        //             } else if (ageNum < 6) {
        //                 min = 80D;
        //                 max = 130D;
        //                 suggest = "年龄3-5岁心率正常值为80-130bpm";
        //             } else if (ageNum < 11) {
        //                 min = 70D;
        //                 max = 120D;
        //                 suggest = "年龄6-10岁心率正常值为70-120bpm";
        //             } else if (ageNum < 18) {
        //                 min = 60D;
        //                 max = 120D;
        //                 suggest = "年龄11-17岁心率正常值为60-120bpm";
        //             } else if (ageNum <= 150) {
        //                 min = 60D;
        //                 max = 110D;
        //                 suggest = "年龄18-150岁心率正常值为60-110bpm";
        //             }
        //             int flag = compareValue(min, max, v);
        //             if (flag == 0) {
        //                 String msg = String.format("该患者%s，%s", content, suggest);
        //                 BillMsg billMsg = new BillMsg();
        //                 billMsg.setMsg(msg);
        //                 billMsg.setContent(content);
        //                 otherList.add(billMsg);
        //             }
        //         }
        //     }
        // }
    }

    public int compareValue(Double min, Double max, Double v) {
        if (min == null || max == null || v == null) {
            return -1;
        }
        if (v >= min && v <= max) {
            return 1;
        }
        return 0;
    }

    /**
     * 处理业务——输血
     *
     * @param wordCrfDTO
     * @param res
     */
    public void processTransfusion(WordCrfDTO wordCrfDTO, IndicationDTO res) {
        List<Lis> lisList = wordCrfDTO.getLis();
        List<BillMsg> otherList = res.getOtherList();
        // 1、化验为key
        for (Lis lis : lisList) {
            RuleVO ruleVO = getRuleVO(lis, RuleTypeEnum.transfusion.getKey());
            // 1、获取相关数据
            if (ruleVO != null) {
                RuleExtDTO ruleExtDTO = commonFacade.getRuleData(ruleVO);
                if (ruleExtDTO == null) {
                    continue;
                }
                List<RuleConditionDTO> ruleConditionDTOList = ruleExtDTO.getRuleConditionDTOList();
                for (RuleConditionDTO ruleConditionDTO : ruleConditionDTOList) {
                    List<RuleBaseDTO> ruleBaseDTOList = ruleConditionDTO.getRuleBaseDTOList();
                    // 2、规则匹配
                    lisRule.otherTransfusion(lis, ruleBaseDTOList, ruleConditionDTO.getMsg(), otherList, wordCrfDTO);
                }
            }
        }

        // 2、输血为key
        List<ItemExt> transfusionList = wordCrfDTO.getTransfusion();
        if (ListUtil.isNotEmpty(transfusionList)) {
            Long dateInterval = redisUtil.getByKeyAndField(RedisEnum.dateInterval.getName(), RedisEnum.dateInterval.getField().get(1));
            Long minute = (-1L == dateInterval) ? 99999999999L : 60L * dateInterval;

            for (ItemExt bean : transfusionList) {
                RuleVO ruleVO = getRuleVO(bean, RuleTypeEnum.transfusion.getKey());
                // 1、获取相关数据
                if (ruleVO != null) {
                    RuleExtDTO ruleExtDTO = commonFacade.getRuleData(ruleVO);
                    if (ruleExtDTO == null) {
                        continue;
                    }
                    // int dateFlag = CoreUtil.compareTime(ruleExtDTO.getFinishDateValue(), "2022-01-25 15:00:00", minute, false, false);
                    int dateFlag = CoreUtil.compareTime(ruleExtDTO.getFinishDateValue(), DateUtil.formatDateTime(DateUtil.now()), minute, false, false);
                    if (dateFlag != 1) {
                        continue;
                    }
                    List<RuleConditionDTO> ruleConditionDTOList = ruleExtDTO.getRuleConditionDTOList();
                    for (RuleConditionDTO ruleConditionDTO : ruleConditionDTOList) {
                        List<RuleBaseDTO> ruleBaseDTOList = ruleConditionDTO.getRuleBaseDTOList();
                        // 2、规则匹配
                        lisRule.otherTransfusionLis(bean, ruleBaseDTOList, ruleConditionDTO.getMsg(), otherList, wordCrfDTO, minute);
                    }
                }
            }
        }
    }

    /**
     * 处理业务——特殊规则
     *
     * @param wordCrfDTO
     * @param res
     */
    public void processSpecial(WordCrfDTO wordCrfDTO, IndicationDTO res) {
        List<BillMsg> otherList = res.getOtherList();
        // 判断VTE评估结果和出血风险结果
        List<Scale> scaleList = wordCrfDTO.getScale();
        // VTE评估结果
        String vteResult = "";
        // VTE出血风险
        String vteBlood = "";
        if (ListUtil.isNotEmpty(scaleList)) {
            for (Scale scale : scaleList) {
                String result = scale.getResult();
                if (StringUtil.isNotBlank(result)) {
                    if (result.matches("高危|中危|极高危")) {
                        vteResult = result;
                    } else if (result.matches("有|无")) {
                        vteBlood = result;
                    }
                }
            }
        }

        // VTE评估结果 和 VTE出血风险都有值
        if (StringUtil.isNotBlank(vteResult) && StringUtil.isNotBlank(vteBlood)) {
            // 是否有药物医嘱
            Boolean drugAdviceFlag = false;
            // 是否有机械医嘱
            Boolean mechanicalAdviceFlag = false;
            // 获取药品医嘱
            List<String> drugAdviceList = redisUtil.get(RedisEnum.vteAdvice.getName() + "药物医嘱");
            drugAdviceFlag = hasAdvice(wordCrfDTO.getDrug(), drugAdviceList);
            if (!drugAdviceFlag) {
                // 获取机械医嘱
                List<String> mechanicalAdviceList = redisUtil.get(RedisEnum.vteAdvice.getName() + "机械医嘱");
                mechanicalAdviceFlag = hasAdvice(wordCrfDTO.getOtherAdvice(), mechanicalAdviceList);
            }
            // 无药物医嘱且无机械医嘱，拼接提示语
            if (!drugAdviceFlag && !mechanicalAdviceFlag) {
                String msg = "该患者VTE评估为%s，伴有出血风险，建议开机械预防医嘱";
                if (vteBlood.matches("无")) {
                    msg = "该患者VTE评估为%s，无出血风险，建议开药物或机械预防医嘱";
                }
                msg = String.format(msg, vteResult);
                BillMsg billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.other.getName(), msg, null, null, null);
                CoreUtil.addBeanToList(otherList, billMsg);
            }
        }
    }

    /**
     * 是否有效医嘱(6天)
     *
     * @param tList
     * @param nameList
     * @param <T>
     * @return
     */
    public <T> Boolean hasAdvice(List<T> tList, List<String> nameList) {
        Boolean hasAdviceFlag = false;
        if (ListUtil.isEmpty(tList) || ListUtil.isEmpty(nameList)) {
            return hasAdviceFlag;
        }
        for (T item : tList) {
            String name = ReflectUtil.getProperty(item, "name");
            if (StringUtil.isNotBlank(name)) {
                if (nameList.contains(name)) {
                    // 开单时间
                    String dateValue = ReflectUtil.getProperty(item, "dateValue");
                    // 当前时间
                    String curDate = DateUtil.formatDateTime(new Date());
                    // 6天有效医嘱
                    int flag = CoreUtil.compareTime(dateValue, curDate, 60L * 24 * 6, true, false);
                    if (flag == 1) {
                        hasAdviceFlag = true;
                        break;
                    }
                }
            }
        }
        return hasAdviceFlag;
    }


    /**
     * 获取其他值提醒入参——辅检
     *
     * @param pacsLabel
     * @return
     */
    public List<RuleVO> getRuleVO(PacsLabel pacsLabel) {
        List<RuleVO> ruleVOList = Lists.newArrayList();

        // 辅检结果
        List<Item> itemList = pacsLabel.getRes();
        if (ListUtil.isNotEmpty(itemList)) {
            for (Item bean : itemList) {
                if (StringUtil.isNotBlank(bean.getUniqueName())) {
                    RuleVO ruleVO = new RuleVO();
                    ruleVO.setInputName(bean.getName());
                    ruleVO.setLibName(bean.getUniqueName());
                    ruleVO.setLibType(LexiconEnum.PacsResult.getKey());
                    ruleVO.setLibTypeName(TypeEnum.pacs.getName());
                    ruleVO.setDateValue(bean.getDateValue());
                    ruleVO.setSuffix(String.valueOf(RuleTypeEnum.other.getKey()));
                    ruleVOList.add(ruleVO);
                }
            }
        }

        // 辅检描述
        List<PacsNum> pacsNumList = pacsLabel.getPacsNumList();
        if (ListUtil.isNotEmpty(pacsNumList)) {
            for (PacsNum pacsNum : pacsNumList) {
                RuleVO ruleVO = new RuleVO();
                ruleVO.setInputName(pacsNum.getName());
                ruleVO.setLibName(pacsNum.getStandName());
                ruleVO.setLibType(LexiconEnum.PacsDescribe.getKey());
                ruleVO.setLibTypeName(TypeEnum.pacs.getName());
                ruleVO.setSuffix(String.valueOf(RuleTypeEnum.other.getKey()));
                ruleVOList.add(ruleVO);
            }
        }
        return ruleVOList;
    }

    /**
     * 获取其他值提醒入参——化验
     *
     * @param lis
     * @return
     */
    public RuleVO getRuleVO(Lis lis, Integer key) {
        if (StringUtil.isNotBlank(lis.getUniqueName())) {
            RuleVO ruleVO = new RuleVO();
            ruleVO.setInputName(lis.getName());
            ruleVO.setLibName(lis.getUniqueName());
            ruleVO.setLibType(LexiconEnum.LisSubName.getKey());
            ruleVO.setLibTypeName(TypeEnum.lis.getName());
            ruleVO.setDateValue(lis.getDateValue());
            ruleVO.setSuffix(String.valueOf(key));
            return ruleVO;
        }
        return null;
    }

    /**
     * 获取其他值提醒入参——输血
     *
     * @param i
     * @return
     */
    public RuleVO getRuleVO(ItemExt i, Integer key) {
        if (StringUtil.isNotBlank(i.getUniqueName())) {
            RuleVO ruleVO = new RuleVO();
            ruleVO.setInputName(i.getName());
            ruleVO.setLibName(i.getUniqueName());
            ruleVO.setLibType(LexiconEnum.Transfusion.getKey());
            ruleVO.setLibTypeName(LexiconEnum.Transfusion.getName());
            ruleVO.setDateValue(i.getDateValue());
            ruleVO.setFinishDateValue(i.getFinishDateValue());
            ruleVO.setSuffix(String.valueOf(key));
            return ruleVO;
        }
        return null;
    }
}
