package com.diagbot.rule;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleSimpleDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.ConEnum;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RegexUtil;
import com.diagbot.util.StringUtil;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @description: 人群规则
 * @author: zhoutg
 * @time: 2020/8/3 14:47
 */
@Component
public class GroupRule {

    @Autowired
    CommonRule commonRule;

    /**
     * 禁忌人群开单项
     *
     * @param wordCrfDTO
     * @param ruleBaseDTO
     * @param ruleSimpleDTO
     */
    public Boolean bill(WordCrfDTO wordCrfDTO, RuleBaseDTO ruleBaseDTO, RuleSimpleDTO ruleSimpleDTO) {
        Map map = rule(wordCrfDTO, ruleBaseDTO);
        boolean flag = CoreUtil.getMapFlag(map);
        if (flag) {
            List<String> contentList = Lists.newArrayList();
            if (!contentList.contains(ruleBaseDTO.getBaseLibName())) {
                contentList.add(ruleBaseDTO.getBaseLibName());
            }
            CoreUtil.setListAndType(ruleSimpleDTO, contentList, ConEnum.group.getName());
        }
        return flag;
    }

    /**
     * 人群推送
     *
     * @param wordCrfDTO
     * @param ruleBaseDTO
     * @param baseIdList
     * @param ids
     */
    public void push(WordCrfDTO wordCrfDTO, RuleBaseDTO ruleBaseDTO, List<Long> baseIdList, String ids) {
        Map map = rule(wordCrfDTO, ruleBaseDTO);
        if (CoreUtil.getMapFlag(map)) {
            CoreUtil.addSplitString(baseIdList, ids);
        }
    }

    /**
     * 人群规则判断
     *
     * @param wordCrfDTO
     * @param ruleBaseDTO
     * @return
     */
    public Map rule(WordCrfDTO wordCrfDTO, RuleBaseDTO ruleBaseDTO) {
        Map res = new LinkedHashMap();
        Double age = wordCrfDTO.getAgeNum();
        Integer sex = wordCrfDTO.getSex();
        // 妊娠目前从诊断里判断,包含“妊娠”
        String symptom = wordCrfDTO.getSymptom(); // 现病史内容
        String menstrual = wordCrfDTO.getMenstrual(); // 月经史
        String marriage = wordCrfDTO.getMarriage(); // 婚姻
        String marital = wordCrfDTO.getMarital();// 婚育史
        String pasts = wordCrfDTO.getPasts(); // 既往史
        boolean flag = false;
        String regex = "";
        String regex2 = "";
        List<String> groupList = Lists.newArrayList(); // 分组信息
        List<String> nameList = CoreUtil.getSplit(ruleBaseDTO.getBaseLibName());
        for (String name : nameList) {
            switch (name) {
                case "妊娠":
                    Map map = gravidityRule(wordCrfDTO);
                    if (CoreUtil.getMapFlag(map) == true) {
                        flag = true;
                    }
                    break;
                case "流产":
                    flag = commonRule.containsItem(wordCrfDTO.getDiag(), name);
                    break;
                case "月经期":
                    regex = "月经第(([零一二三四五六七八九十]{0,3})||([0-9]{0,2}))天";
                    // regex2 = "(?<!上一次)经期";
                    if (RegexUtil.getRegexRes(symptom, regex) || RegexUtil.getRegexRes(menstrual, regex)
                            // || RegexUtil.getRegexRes(symptom, regex2) || RegexUtil.getRegexRes(menstrual, regex2)
                    ) {
                        flag = true;
                    }
                    break;
                case "幼儿":
                    if (age != null) {
                        if (age < 6.0D && 1.0D <= age) {
                            flag = true;
                        }
                    }
                    break;
                case "儿童":
                    if (age != null) {
                        if (age < 18.0D && age >= 6.0D) {
                            flag = true;
                        }
                    }
                    break;
                case "成人":
                    if (age >= 18) {
                        flag = true;
                    }
                    break;
                case "新生儿":
                    if (age <= 1) {
                        flag = true;
                    }
                    break;
                case "未婚女性":
                    regex = "未婚";
                    if (sex == 2 &&
                            (RegexUtil.getRegexRes(marriage, regex) || "2".equals(marriage) || RegexUtil.getRegexRes(marital, regex))
                    ) {
                        flag = true;
                    }
                    break;
                case "未经产妇": // 女性+（婚育史或既往史)数值累加为0
                    regex = "(\\d+)-(\\d+)-(\\d+)-(\\d+)";
                    if (sex == 2) {
                        if (StringUtil.isNotBlank(marital) && RegexUtil.getRegexRes(marital, regex)) {
                            // 婚育史判断
                            groupList = RegexUtil.getRegexDataList(marital, regex);
                        } else if (StringUtil.isNotBlank(pasts) && RegexUtil.getRegexRes(pasts, regex)) {
                            // 既往史判断
                            groupList = RegexUtil.getRegexDataList(pasts, regex);
                        }
                        if (ListUtil.isNotEmpty(groupList) && groupList.size() == 4) {
                            if (Integer.parseInt(groupList.get(0)) + Integer.parseInt(groupList.get(1))
                                    + Integer.parseInt(groupList.get(2)) == 0) {
                                flag = true;
                            }
                        }
                    }
                    break;
                case "经产妇": // 女性+（婚育史或既往史)数值累加>=1
                    regex = "(\\d+)-(\\d+)-(\\d+)-(\\d+)";
                    if (sex == 2) {
                        if (StringUtil.isNotBlank(marital) && RegexUtil.getRegexRes(marital, regex)) {
                            // 婚育史判断
                            groupList = RegexUtil.getRegexDataList(marital, regex);
                        } else if (StringUtil.isNotBlank(pasts) && RegexUtil.getRegexRes(pasts, regex)) {
                            // 既往史判断
                            groupList = RegexUtil.getRegexDataList(pasts, regex);
                        }
                        if (ListUtil.isNotEmpty(groupList) && groupList.size() == 4) {
                            if (Integer.parseInt(groupList.get(0)) + Integer.parseInt(groupList.get(1))
                                    + Integer.parseInt(groupList.get(2)) >= 1) {
                                flag = true;
                            }
                        }
                    }
                    break;
                default:
                    break;
            }
        }
        res.put("flag", flag);
        res.put("msg", ruleBaseDTO.getBaseLibName());
        return res;
    }

    /**
     * 判断妊娠规则
     *
     * @param wordCrfDTO
     * @return
     */
    public Map gravidityRule(WordCrfDTO wordCrfDTO) {
        Map<String, Object> map = new LinkedHashMap<>();
        // 妊娠目前从诊断里判断,包含“妊娠”
        String symptom = wordCrfDTO.getSymptom(); // 现病史内容
        String chief = wordCrfDTO.getChief(); // 主诉
        // 主诉、现病史、诊断包含“孕妇”，“妊娠”
        boolean flag = false;
        if (symptom.contains("孕妇") || chief.contains("孕妇")
                || symptom.contains("妊娠") || chief.contains("妊娠")) {
            flag = true;
        } else {
            // 主诉、现病史出现孕几周
            String regex = ".*孕.{1,4}周.*"; // 孕xx周
            if (symptom.matches(regex) || chief.matches(regex)) {
                flag = true;
            } else {
                List<Item> diag = wordCrfDTO.getDiag();
                for (Item item : diag) {
                    // 诊断出现“孕几周”，孕字必须是第一个
                    String regex1 = "孕.{1,4}周.*"; // 孕xx周
                    if (item.getUniqueName().matches(regex1) || item.getUniqueName().contains("孕妇")
                            || item.getUniqueName().contains("妊娠")) {
                        flag = true;
                        break;
                    }
                }
            }
        }
        if (flag) {
            map.put("flag", flag);
            map.put("msg", Arrays.asList("妊娠"));
        }
        return map;
    }
}
