package com.diagbot.rule;

import com.diagbot.biz.push.entity.Lis;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.LisCtritalDTO;
import com.diagbot.dto.LisOtherDTO;
import com.diagbot.dto.MatchDTO;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleSimpleDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.BaseTypeEnum;
import com.diagbot.enums.ConEnum;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.MsgNewUtil;
import com.diagbot.util.MsgUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.ItemExt;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @description: 化验规则
 * @author: kwz
 * @time: 2020/8/3 14:47
 */
@Component
public class LisRule {

    @Autowired
    GroupRule groupRule;
    @Autowired
    CommonRule commonRule;
    @Autowired
    MsgNewUtil msgNewUtil;
    @Autowired
    SexRule sexRule;
    @Autowired
    AgeRule ageRule;

    /**
     * 比较化验——开单合理性
     *
     * @param inputLis
     * @param ruleBaseDTO
     * @param ruleSimpleDTO
     */
    public Boolean bill(List<Lis> inputLis, RuleBaseDTO ruleBaseDTO, RuleSimpleDTO ruleSimpleDTO) {
        boolean flag = false;
        List<String> contentList = Lists.newArrayList();
        if (ListUtil.isNotEmpty(inputLis) && StringUtil.isNotBlank(ruleBaseDTO.getBaseLibName())) {
            for (Lis lis : inputLis) {
                MatchDTO matchDTO = CoreUtil.compareLis(ruleBaseDTO, lis);
                if (matchDTO != null) {
                    if (!contentList.contains(matchDTO.getContent())) {
                        contentList.add(matchDTO.getContent());
                    }
                    flag = true;
                }
            }
        }
        if (flag) {
            CoreUtil.setListAndType(ruleSimpleDTO, contentList, ConEnum.lis.getName());
        }
        return flag;
    }

    /**
     * 比较化验——高风险手术
     *
     * @param inputLis
     * @param ruleBaseDTO
     * @param highRiskList
     * @param ruleSimpleDTO
     */
    public void highOperation(List<Lis> inputLis, RuleBaseDTO ruleBaseDTO, List<BillMsg> highRiskList, RuleSimpleDTO ruleSimpleDTO) {
        if (ListUtil.isNotEmpty(inputLis) && StringUtil.isNotBlank(ruleBaseDTO.getBaseLibName())) {
            for (Lis lis : inputLis) {
                MatchDTO matchDTO = CoreUtil.compareLis(ruleBaseDTO, lis);
                if (matchDTO != null) {
                    BillMsg billMsg = MsgUtil.getCommonHighRiskMsg(matchDTO.getContent(), ruleSimpleDTO.getInputName(), ruleSimpleDTO.getLibTypeName());
                    highRiskList.add(billMsg);
                }
            }
        }
    }

    /**
     * 比较化验——其他值提醒
     *
     * @param lisBean
     * @param ruleBaseDTOList
     * @param otherList
     * @param wordCrfDTO
     * @param msg
     */
    public void otherLis(Lis lisBean, List<RuleBaseDTO> ruleBaseDTOList, List<BillMsg> otherList, WordCrfDTO wordCrfDTO, String msg) {
        if (ListUtil.isEmpty(ruleBaseDTOList)) {
            return;
        }
        LisOtherDTO lisOtherDTO = new LisOtherDTO();
        for (RuleBaseDTO ruleBaseDTO : ruleBaseDTOList) {
            boolean flag = false;
            if (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType()) != null) { // 避免空指针
                Map<String, Object> map = null;
                switch (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType())) {
                    case LisSubName: // 化验细项
                        if (ruleBaseDTO.getBaseLibName().equals(lisBean.getUniqueName())) {
                            // 化验自身
                            MatchDTO matchDTO = CoreUtil.compareLis(ruleBaseDTO, lisBean);
                            if (matchDTO != null) {
                                lisOtherDTO.setName(matchDTO.getContent());
                                lisOtherDTO.setLisNameDetail(matchDTO.getLisNameDetail());
                                flag = true;
                            }
                        } else {
                            // 其他化验——暂无数据
                            List<MatchDTO> matchDTOList = CoreUtil.compareLis(ruleBaseDTO, wordCrfDTO.getLis());
                            if (ListUtil.isNotEmpty(matchDTOList)) {
                                List<String> msgLis = matchDTOList.stream().map(r -> r.getContent()).collect(Collectors.toList());
                                lisOtherDTO.setLisMsg(msgLis);
                                flag = true;
                            }
                        }
                        break;
                    case Group: // 人群
                        map = groupRule.rule(wordCrfDTO, ruleBaseDTO);
                        flag = CoreUtil.getMapFlag(map);
                        if (flag) {
                            lisOtherDTO.getGroupMsg().add((String) map.get("msg"));
                        }
                        break;
                    case Disease: // 诊断
                        map = commonRule.compareItem(wordCrfDTO.getDiag(), ruleBaseDTO);
                        flag = CoreUtil.getMapFlag(map);
                        if (flag) {
                            lisOtherDTO.setDiseaseMsg((List<String>) map.get("msgList"));
                        }
                        break;
                    case Medicine: // 药品
                        map = commonRule.compareItem(wordCrfDTO.getDrugSource(), ruleBaseDTO);
                        flag = CoreUtil.getMapFlag(map);
                        if (flag) {
                            lisOtherDTO.setDrugMsg((List<String>) map.get("msgList"));
                        }
                        break;
                    case Gender: // 性别
                        flag = sexRule.getFlag(wordCrfDTO, ruleBaseDTO);
                        if (flag) {
                            lisOtherDTO.setSexMsg(ruleBaseDTO.getBaseLibName());
                        }
                        break;
                    default:
                        break;
                }
            }
            // 有任一条件不满足，直接退出
            if (!flag) {
                return;
            }
        }
        // 条件全部符合
        if (StringUtil.isNotBlank(lisOtherDTO.getName())) {
            String lisNameDetail = lisOtherDTO.getLisNameDetail(); // 化验套餐和明细
            // 就化验本身一个条件符合
            if (ruleBaseDTOList.size() == 1) {
                BillMsg billMsg = msgNewUtil.getCommonOtherMsg(TypeEnum.lis.getName(),
                        lisOtherDTO.getName(), "", TypeEnum.lisSelf.getName(), lisBean, msg, lisNameDetail);
                CoreUtil.addBeanToList(otherList, billMsg);
            } else { // 有其他条件符合
                // 诊断
                if (ListUtil.isNotEmpty(lisOtherDTO.getDiseaseMsg())) {
                    for (String str : lisOtherDTO.getDiseaseMsg()) {
                        BillMsg billMsg = msgNewUtil.getCommonOtherMsg(TypeEnum.lis.getName(), lisOtherDTO.getName(),
                                str, TypeEnum.disease.getName(), lisBean, msg, lisNameDetail);
                        CoreUtil.addBeanToList(otherList, billMsg);
                    }
                }
                // 药品
                if (ListUtil.isNotEmpty(lisOtherDTO.getDrugMsg())) {
                    for (String str : lisOtherDTO.getDrugMsg()) {
                        BillMsg billMsg = msgNewUtil.getCommonOtherMsg(TypeEnum.lis.getName(), lisOtherDTO.getName(),
                                str, TypeEnum.drug.getName(), lisBean, msg, lisNameDetail);
                        CoreUtil.addBeanToList(otherList, billMsg);
                    }
                }
                // 人群
                if (ListUtil.isNotEmpty(lisOtherDTO.getGroupMsg())) {
                    for (String str : lisOtherDTO.getGroupMsg()) {
                        BillMsg billMsg = msgNewUtil.getCommonOtherMsg(TypeEnum.lis.getName(), lisOtherDTO.getName(),
                                str, TypeEnum.group.getName(), lisBean, msg, lisNameDetail);
                        CoreUtil.addBeanToList(otherList, billMsg);
                    }
                }
                // 化验——暂无数据
                if (ListUtil.isNotEmpty(lisOtherDTO.getLisMsg())) {
                    for (String str : lisOtherDTO.getLisMsg()) {
                        BillMsg billMsg = msgNewUtil.getCommonOtherMsg(TypeEnum.lis.getName(), lisOtherDTO.getName(),
                                str, TypeEnum.lis.getName(), lisBean, msg, lisNameDetail);
                        CoreUtil.addBeanToList(otherList, billMsg);
                    }
                }
                // 性别
                if (StringUtil.isNotBlank(lisOtherDTO.getSexMsg())) {
                    BillMsg billMsg = msgNewUtil.getCommonOtherMsg(TypeEnum.lis.getName(), lisOtherDTO.getName(),
                            lisOtherDTO.getSexMsg(), TypeEnum.gender.getName(), lisBean, msg, lisNameDetail);
                    CoreUtil.addBeanToList(otherList, billMsg);
                }
            }
        }
    }

    /**
     * 比较化验——其他值提醒
     *
     * @param lisBean
     * @param ruleBaseDTOList
     * @param otherList
     * @param wordCrfDTO
     */
    public void otherTransfusion(Lis lisBean, List<RuleBaseDTO> ruleBaseDTOList, String msg, List<BillMsg> otherList, WordCrfDTO wordCrfDTO) {
        if (ListUtil.isEmpty(ruleBaseDTOList)) {
            return;
        }
        LisOtherDTO lisOtherDTO = new LisOtherDTO();
        lisOtherDTO.setMsg(msg);
        for (RuleBaseDTO ruleBaseDTO : ruleBaseDTOList) {
            boolean flag = false;
            if (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType()) != null) { // 避免空指针
                Map<String, Object> map = null;
                switch (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType())) {
                    case LisSubName: // 化验细项
                        if (ruleBaseDTO.getBaseLibName().equals(lisBean.getUniqueName())) {
                            // 化验自身
                            MatchDTO matchDTO = CoreUtil.compareLis(ruleBaseDTO, lisBean);
                            if (matchDTO != null) {
                                lisOtherDTO.setName(matchDTO.getContent());
                                lisOtherDTO.setLisNameDetail(matchDTO.getLisNameDetail());
                                flag = true;
                            }
                        } else {
                            // 其他化验——暂无数据
                            List<MatchDTO> matchDTOList = CoreUtil.compareLis(ruleBaseDTO, wordCrfDTO.getLis());
                            if (ListUtil.isNotEmpty(matchDTOList)) {
                                List<String> msgLis = matchDTOList.stream().map(r -> r.getContent()).collect(Collectors.toList());
                                lisOtherDTO.setLisMsg(msgLis);
                                flag = true;
                            }
                        }
                        break;
                    case Disease: // 诊断
                        map = commonRule.compareItem(wordCrfDTO.getDiag(), ruleBaseDTO);
                        flag = CoreUtil.getMapFlag(map);
                        if (flag) {
                            lisOtherDTO.setDiseaseMsg((List<String>) map.get("msgList"));
                        }
                        break;
                    case Medicine: // 药品
                        map = commonRule.compareItem(wordCrfDTO.getDrugSource(), ruleBaseDTO);
                        flag = CoreUtil.getMapFlag(map);
                        if (flag) {
                            lisOtherDTO.setDrugMsg((List<String>) map.get("msgList"));
                        }
                    case Operation: // 手术
                        map = commonRule.compareItem(wordCrfDTO.getOperation(), ruleBaseDTO);
                        flag = CoreUtil.getMapFlag(map);
                        if (flag) {
                            lisOtherDTO.setOperationMsg((List<String>) map.get("msgList"));
                        }
                        break;
                    default:
                        break;
                }
            }
            // 有任一条件不满足，直接退出
            if (!flag) {
                return;
            }
        }
        // 条件全部符合
        if (StringUtil.isNotBlank(lisOtherDTO.getName())) {
            // 化验本身符合，其他不符合
            if (ListUtil.isEmpty(lisOtherDTO.getDiseaseMsg())
                    && ListUtil.isEmpty(lisOtherDTO.getOperationMsg())
                    && ListUtil.isEmpty(lisOtherDTO.getLisMsg())) {
                BillMsg billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.transfusion.getName(),
                        lisOtherDTO.getName(), msg, TypeEnum.lis.getName(), lisBean);
                otherList.add(billMsg);
            } else {
                // 诊断
                if (ListUtil.isNotEmpty(lisOtherDTO.getDiseaseMsg())) {
                    for (String str : lisOtherDTO.getDiseaseMsg()) {
                        BillMsg billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.transfusion.getName(),
                                "诊断" + str + "，" + lisOtherDTO.getName(), msg, TypeEnum.disease.getName(), lisBean);
                        otherList.add(billMsg);
                    }
                }
                // 手术
                if (ListUtil.isNotEmpty(lisOtherDTO.getOperationMsg())) {
                    for (String str : lisOtherDTO.getOperationMsg()) {
                        BillMsg billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.transfusion.getName(),
                                lisOtherDTO.getName() + "，已开" + str, msg, TypeEnum.operation.getName(), lisBean);
                        otherList.add(billMsg);
                    }
                }
                // 化验化验本身符合，其他不符合
                if (ListUtil.isNotEmpty(lisOtherDTO.getLisMsg())) {
                    for (String str : lisOtherDTO.getLisMsg()) {
                        BillMsg billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.transfusion.getName(),
                                lisOtherDTO.getName() + "，" + str, msg, TypeEnum.lis.getName(), lisBean);
                        otherList.add(billMsg);
                    }
                }
                // // 药品暂未用
                // if (ListUtil.isNotEmpty(lisOtherDTO.getDrugMsg())) {
                //     for (String str : lisOtherDTO.getDrugMsg()) {
                //         BillMsg billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.transfusion.getName(),
                //                 lisOtherDTO.getName(), msg, TypeEnum.drug.getName(), lisBean);
                //         otherList.add(billMsg);
                //     }
                // }
            }
        }
    }


    /**
     * 比较化验是否存在——其他值提醒输血
     *
     * @param bean
     * @param ruleBaseDTOList
     * @param msg
     * @param wordCrfDTO
     * @param otherList
     * @param minute
     */
    public void otherTransfusionLis(ItemExt bean, List<RuleBaseDTO> ruleBaseDTOList, String msg, List<BillMsg> otherList, WordCrfDTO wordCrfDTO, Long minute) {
        if (ListUtil.isEmpty(ruleBaseDTOList)) {
            return;
        }
        for (RuleBaseDTO ruleBaseDTO : ruleBaseDTOList) {
            boolean flag = false;
            if (BaseTypeEnum.getEnum(ruleBaseDTO.getBaseType()) != null) {// 避免空指针
                switch (BaseTypeEnum.getEnum(ruleBaseDTO.getBaseType())) {
                    case B7: // 化验历史开单项匹配
                        flag = CoreUtil.hasLis(ruleBaseDTO, wordCrfDTO.getLis(), bean.getFinishDateValue(), minute);
                        break;
                    case B8: // 化验当前开单项匹配
                        flag = CoreUtil.hasLis(ruleBaseDTO, wordCrfDTO.getLisOrder(), bean.getFinishDateValue(), minute);
                        break;
                    default:
                        break;
                }
            }
            // 有任一条件满足，直接退出
            if (flag) {
                return;
            }
        }

        BillMsg billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.other.getName(),
                msg, msg, TypeEnum.transfusion.getName(), null);
        otherList.add(billMsg);
    }


    /**
     * 比较化验——危急值（多个条件）
     *
     * @param wordCrfDTO
     * @param ruleBaseDTOList
     * @param criticalList
     */
    public void critical(Lis lisBean, List<RuleBaseDTO> ruleBaseDTOList, List<BillMsg> criticalList, WordCrfDTO wordCrfDTO, String msg) {
        if (ListUtil.isEmpty(ruleBaseDTOList)) {
            return;
        }

        LisCtritalDTO lisCtritalDTO = new LisCtritalDTO();
        for (RuleBaseDTO ruleBaseDTO : ruleBaseDTOList) {
            boolean flag = false;
            if (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType()) != null) { // 避免空指针
                Map<String, Object> map = null;
                switch (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType())) {
                    case LisSubName: // 化验细项
                        if (ruleBaseDTO.getBaseLibName().equals(lisBean.getUniqueName())) {
                            // 化验自身 【第一个】
                            MatchDTO matchDTO = CoreUtil.compareLis(ruleBaseDTO, lisBean);
                            if (matchDTO != null) {
                                lisCtritalDTO.setName(matchDTO.getContent());
                                flag = true;
                            }
                        } else {
                            // 其他化验
                            List<MatchDTO> matchDTOList = CoreUtil.compareLis(ruleBaseDTO, wordCrfDTO.getLis());
                            if (ListUtil.isNotEmpty(matchDTOList)) {
                                flag = true;
                            }
                        }
                        break;
                    case Group: // 人群
                        map = groupRule.rule(wordCrfDTO, ruleBaseDTO);
                        flag = CoreUtil.getMapFlag(map);
                        if (flag) {
                            lisCtritalDTO.setGroupMsg(ruleBaseDTO.getBaseLibName());
                        }
                        break;
                    case Age: // 年龄
                        flag = ageRule.getFlag(wordCrfDTO, ruleBaseDTO);
                        if (flag) {
                            lisCtritalDTO.setAgeMsg(wordCrfDTO.getAge());
                        }
                        break;
                    default:
                        break;
                }
            }
            // 有任一条件不满足，直接退出
            if (!flag) {
                return;
            }
        }
        if (StringUtil.isNotBlank(lisCtritalDTO.getName())) {
            // 就化验本身一个条件符合
            if (ruleBaseDTOList.size() == 1) {
                BillMsg billMsg = msgNewUtil.getCriticalMsg(TypeEnum.lis.getName(),
                        lisCtritalDTO.getName(), "", TypeEnum.lisSelf.getName(), msg);
                CoreUtil.addBeanToList(criticalList, billMsg);
            } else { // 有其他条件符合
                // 化验——暂无数据
                if (ListUtil.isNotEmpty(lisCtritalDTO.getLisMsg())) {
                    for (String str : lisCtritalDTO.getLisMsg()) {
                        BillMsg billMsg = msgNewUtil.getCriticalMsg(TypeEnum.lis.getName(), lisCtritalDTO.getName(),
                                str, TypeEnum.lis.getName(), msg);
                        CoreUtil.addBeanToList(criticalList, billMsg);
                    }
                }
                // 人群
                if (StringUtil.isNotBlank(lisCtritalDTO.getGroupMsg())) {
                    BillMsg billMsg = msgNewUtil.getCriticalMsg(TypeEnum.lis.getName(), lisCtritalDTO.getName(),
                            lisCtritalDTO.getGroupMsg(), TypeEnum.group.getName(), msg);
                    CoreUtil.addBeanToList(criticalList, billMsg);
                }
                // 年龄
                if (StringUtil.isNotBlank(lisCtritalDTO.getAgeMsg())) {
                    BillMsg billMsg = msgNewUtil.getCriticalMsg(TypeEnum.lis.getName(), lisCtritalDTO.getName(),
                            lisCtritalDTO.getAgeMsg(), TypeEnum.age.getName(), msg);
                    CoreUtil.addBeanToList(criticalList, billMsg);
                }
            }
        }
    }

    /**
     * 比较化验——推理
     *
     * @param inputLis
     * @param ruleBaseDTO
     * @param baseIdList
     * @param ids
     */
    public void push(List<Lis> inputLis, RuleBaseDTO ruleBaseDTO, List<Long> baseIdList, String ids) {
        if (ListUtil.isNotEmpty(inputLis) && StringUtil.isNotBlank(ruleBaseDTO.getBaseLibName())) {
            for (Lis lis : inputLis) {
                MatchDTO matchDTO = CoreUtil.compareLis(ruleBaseDTO, lis);
                if (matchDTO != null) {
                    CoreUtil.addSplitString(baseIdList, ids);
                }
            }
        }
    }
}
