package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.CacheFacade;
import com.diagbot.facade.ConceptInfoFacade;
import com.diagbot.util.RedisUtil;
import com.diagbot.vo.DecodeRedisVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * <p>
 * 缓存 前端控制器
 * </p>
 *
 * @author zhoutg
 * @since 2018-12-25
 */
@RequestMapping("/cache")
@RestController
@SuppressWarnings("unchecked")
@Api(value = "缓存API", tags = { "缓存API" })
@Slf4j
public class CacheController {

    @Autowired
    CacheFacade cacheFacade;
    @Autowired
    private ConceptInfoFacade conceptInfoFacade;
    @Autowired
    RedisUtil redisUtil;

    @ApiOperation(value = "清除启动加载类缓存[by:zhoutg]",
            notes = "")
    @PostMapping("/clear")
    @SysLogger("clear")
    public RespDTO<Boolean> clear() {
        cacheFacade.clear();
//        cacheFacade.loadDrugTypeCache();
        cacheFacade.getDiseaseCorrespondCache();
        cacheFacade.getdiseaseFilterCache();
        cacheFacade.loadPushModel(); // 重新加载推送类型
        cacheFacade.loadDeptPush(); // 重新加载标准科室和推送类型的映射关系
        cacheFacade.loadFrequency(); // 重新加载频次
        conceptInfoFacade.loadCustomDictionary(); // 重新加载NLP分词字典
        cacheFacade.loadMsg(); // 加载开单提示语
        cacheFacade.loadVteAdviceCache(); // 重新加载vte医嘱数据
        cacheFacade.loadDateInterval(); // 重新加载开单时间间隔成功
        cacheFacade.clearStandConvert(); // 清除标准词转换缓存
        cacheFacade.loadAllBaseDiagnoseCache(); // 重新加载诊断依据
        cacheFacade.loadAllRuleCache(); // 重新加载开单规则
        return RespDTO.onSuc(true);
    }

    @ApiOperation(value = "redis缓存反解[by:zhoutg]",
            notes = "key：缓存key<br>" +
                    "field：指定属性")
    @PostMapping("/decodeRedis")
    @SysLogger("decodeRedis")
    public RespDTO<Object> decodeRedis(@RequestBody @Valid DecodeRedisVO decodeRedisVO) {
        Object data = cacheFacade.decodeRedis(decodeRedisVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "重新加载基础诊断依据[by:zhoutg]",
            notes = "")
    @PostMapping("/reloadDiagnose")
    @SysLogger("reloadDiagnose")
    public RespDTO<Boolean> reloadDiagnose() {
        cacheFacade.loadAllBaseDiagnoseCache();
        return RespDTO.onSuc(true);
    }

    // @ApiOperation(value = "重新加载基础规则[by:zhoutg]",
    //         notes = "")
    // @PostMapping("/reloadRegulation")
    // @SysLogger("reloadRegulation")
    // public RespDTO<Boolean> reloadRegulation() {
    //     cacheFacade.loadAllBaseRegulationCache();
    //     return RespDTO.onSuc(true);
    // }

    @ApiOperation(value = "重新加载药物缓存[by:zhoutg]",
            notes = "")
    @PostMapping("/clearDrug")
    @SysLogger("clearDrug")
    public RespDTO<Boolean> clearDrug() {
        cacheFacade.loadDrugTypeCache();
        return RespDTO.onSuc(true);
    }

    @ApiOperation(value = "清除标准词转换缓存[by:zhoutg]",
            notes = "")
    @PostMapping("/clearStandConvert")
    @SysLogger("clearStandConvert")
    public RespDTO<Boolean> clearStandConvert() {
        cacheFacade.clearStandConvert();
        return RespDTO.onSuc(true);
    }

    @ApiOperation(value = "清除规则缓存[by:wangfeng]",
            notes = "")
    @PostMapping("/clearRuleInfoAll")
    @SysLogger("clearRuleInfoAll")
    public RespDTO<Boolean> clearRuleInfoAlls() {
        cacheFacade.loadAllRuleCache();
        return RespDTO.onSuc(true);
    }

    @ApiOperation(value = "重新加载大数据缓存[by:zhoutg]",
            notes = "")
    @PostMapping("/reloadPushDB")
    @SysLogger("reloadPushDB")
    public RespDTO<Boolean> reloadPushDB() {
        boolean flag = cacheFacade.reloadPushDB();
        if (flag) {
            return RespDTO.onSuc(flag);
        } else {
            return RespDTO.onError("加载大数据缓存失败");
        }
    }

    @ApiOperation(value = "重新加载标准词及规则、药物缓存、基础诊断依据[by:wangfeng]",
            notes = "")
    @PostMapping("/clearStandRuleDrug")
    @SysLogger("clearStandRuleDrug")
    public RespDTO<Boolean> clearStandRuleDrugAll() {
        cacheFacade.clearStandConvert();
        cacheFacade.loadAllRuleCache();
//        cacheFacade.loadDrugTypeCache();
        cacheFacade.loadAllBaseDiagnoseCache();
        // 加载大数据缓存
        boolean flag = cacheFacade.reloadPushDB();
        if (flag) {
            log.error("加载大数据缓存成功");
            return RespDTO.onSuc(flag);
        } else {
            log.error("加载大数据缓存失败");
            return RespDTO.onError("加载知识库缓存成功，加载大数据缓存失败");
        }
    }

    @ApiOperation(value = "加载推送类型[by:zhoutg]",
            notes = "表：tran_hospital_info，字段：push_mode")
    @PostMapping("/loadPushModel")
    @SysLogger("loadPushModel")
    public RespDTO<Boolean> loadPushModel() {
        cacheFacade.loadPushModel();
        return RespDTO.onSuc(true);
    }

    @ApiOperation(value = "加载标准科室和推送类型的映射关系【sys_dept_push】",
            notes = "")
    @PostMapping("/loadDeptPush")
    @SysLogger("loadDeptPush")
    public RespDTO<Boolean> loadDeptPush() {
        cacheFacade.loadDeptPush();
        return RespDTO.onSuc(true);
    }

    @ApiOperation(value = "重新加载NLP分词字典[by:zhoutg]",
            notes = "")
    @PostMapping("/reloadCustomDictionary")
    @SysLogger("reloadCustomDictionary")
    public RespDTO<Boolean> reloadCustomDictionary() {
        conceptInfoFacade.loadCustomDictionary();
        return RespDTO.onSuc(true);
    }

    @ApiOperation(value = "重新加载开单频次[by:zhoutg]",
            notes = "")
    @PostMapping("/reloadFrequency")
    @SysLogger("reloadFrequency")
    public RespDTO<Boolean> reloadFrequency() {
        cacheFacade.loadFrequency();
        return RespDTO.onSuc(true);
    }
}
